﻿using System;
using System.ComponentModel;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using PowernapPeerWPF.ServiceReference1;

namespace PowernapPeerWPF
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private IPAddress ServerIpAddress { get; }
        private int ServerPort { get; }

        private string SharedFolder { get; }

        public MainWindow()
        {
            string localHostName = Dns.GetHostName();
            //IPHostEntry s = Dns.GetHostEntry(localHostName);
            //ServerIpAddress = s.AddressList[0];

            ServerIpAddress = Dns.GetHostEntry(localHostName)
                .AddressList
                .First(ip => ip.AddressFamily == AddressFamily.InterNetwork);
            // IPv4 http://stackoverflow.com/questions/2048132/net-ipaddress-ipv4


            ServerPort = 14593;

            InitializeComponent();
            SetUpTraceListener();
            DestinationInput.Text = "C:/temp3/";

            SharedFolder = DestinationInput.Text;

            ServerPart();
        }

        private void SetUpTraceListener()
        {
            Trace.AutoFlush = true;
            TraceListener textBoxTraceListener = new TextBoxTraceListener(Dispatcher, StatusOutput);
            Trace.Listeners.Add(textBoxTraceListener);
        }

        private void GetItClicked(object sender, RoutedEventArgs e)
        {
            string filename = FilenameInput.Text;
            Trace.WriteLine(filename);
            using (RepositoryServiceClient client = new RepositoryServiceClient())
            {
                try
                {
                    Task.Run(() => // to make the GUi more responsive
                    {
                        Destination[] destinations = client.Get(filename);
                        Trace.WriteLine("We have some locations " + destinations.Length);
                        if (destinations.Length == 0)
                        {
                            Trace.WriteLine("No locations");
                            return;
                        }
                        foreach (Destination destination in destinations)
                        {
                            Trace.WriteLine(destination.Host + " " + destination.Port);
                        }
                        Destination location = destinations[0];
                        Trace.WriteLine("Chosen location " + location.Host + " " + location.Port);
                        //string substring = location.Host.Substring(0, location.Host.Length - 2);
                        //Trace.WriteLine(substring);
                        using (TcpClient tcpClient = new TcpClient(location.Host, location.Port))
                        {
                            Stream stream = tcpClient.GetStream();
                            StreamWriter writer = new StreamWriter(stream);
                            writer.WriteLine(filename);
                            writer.Flush();
                            //writer.Close();
                            FileStream fileStream = new FileStream(DestinationInput.Text + "copy" + filename, FileMode.Create);
                            stream.CopyTo(fileStream);
                        }
                    });
                }
                catch (Exception ex)
                {
                    Trace.WriteLine(ex.Message);
                }
            }
        }

        private void ServerPart()
        {
            TcpListener tcpListener = new TcpListener(ServerIpAddress, ServerPort);
            tcpListener.Start();
            Trace.WriteLine("TcpListener: " + tcpListener.LocalEndpoint);
            RegisterFiles(SharedFolder, ServerIpAddress, ServerPort);
            Task.Run(() => RunServerPart(tcpListener, SharedFolder));
        }

        private static void RegisterFiles(string path, IPAddress ipAddress, int port)
        {
            string[] files = Directory.GetFiles(path);
            Trace.WriteLine(string.Join("\n", files));
            for (int i = 0; i < files.Length; i++)
            {
                files[i] = GetFileName(files[i]);
            }
            Trace.WriteLine(string.Join("\n", files));
            using (RepositoryServiceClient client = new RepositoryServiceClient())
            {
                // Todo ipAddress.ToString() working??
                // Todo alternative: Use localHostName
                int howMany = client.AddAll(files, ipAddress.ToString(), port);
                Trace.WriteLine("Files registred: " + howMany);
            }
        }

        private static string GetFileName(string path)
        {
            int index = path.LastIndexOf("/");
            return path.Substring(index + 1);
        }

        private static void RunServerPart(TcpListener tcpListener, string path)
        {
            while (true)
            {
                Trace.WriteLine("waiting for client " + tcpListener.LocalEndpoint);
                TcpClient client = tcpListener.AcceptTcpClient();
                Trace.WriteLine("new client connection " + client.Client.RemoteEndPoint);
                Stream stream = client.GetStream();
                StreamReader reader = new StreamReader(stream);
                string request = reader.ReadLine();
                string filename = path + request;
                try
                {
                    using (FileStream fileStream = new FileStream(filename, FileMode.Open))
                    {
                        string response = "200 OK\n";
                        byte[] buffer = Encoding.ASCII.GetBytes(response);
                        stream.Write(buffer, 0, buffer.Length);
                        fileStream.CopyTo(stream);
                        stream.Flush();
                    }
                }
                catch (FileNotFoundException)
                {
                    string response = "404 Not found: " + request + "\n";
                    byte[] buffer = Encoding.ASCII.GetBytes(response);
                    stream.Write(buffer, 0, buffer.Length);
                    stream.Flush();
                }
                client.Close();
            }
        }

        private void MainWindow_OnClosing(object sender, CancelEventArgs e)
        {
            using (RepositoryServiceClient client = new RepositoryServiceClient())
            {
                client.RemoveAll(ServerIpAddress.ToString(), ServerPort);
            }
        }
    }
}
